"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.eslintConfigStarterCreator = void 0;
const creator_utils_1 = require("@knapsack/creator-utils");
const creator_1 = require("@knapsack/prettier-config/creator");
const path_1 = require("path");
const { pkg } = (0, creator_utils_1.findUpPkgJson)(__dirname);
exports.eslintConfigStarterCreator = (0, creator_utils_1.createCreator)({
    id: 'eslint-config-starter',
    getQuestions: async () => ({
        hasReact: {
            type: 'boolean',
            title: 'Using React?',
        },
        hasAva: {
            type: 'boolean',
            title: 'Using Ava for testing?',
        },
        addNpmScript: {
            type: 'boolean',
            title: 'Add lint command to npm scripts?',
        },
        addIgnore: {
            type: 'boolean',
            title: 'Add .prettierignore file?',
            default: false,
        },
    }),
    getTasks: async ({ answers: { hasAva, hasReact, addNpmScript, addIgnore }, config, }) => {
        const { dest } = config;
        const configs = [
            hasReact && '@knapsack/eslint-config-starter/react',
            hasAva && '@knapsack/eslint-config-starter/ava',
            '@knapsack/eslint-config-starter',
        ].filter(Boolean);
        return [
            {
                title: 'Add .eslintrc.js',
                skip: (0, creator_utils_1.hasFileUp)({
                    cwd: dest,
                    file: [
                        '.eslintrc.js',
                        '.eslintrc.cjs',
                        '.eslintrc.yaml',
                        '.eslintrc.yml',
                        '.eslintrc.json',
                        '.eslintrc',
                    ],
                }) && 'Found pre-existing file',
                task: async (ctx, task) => {
                    return creator_utils_1.tasks.addFile({
                        destination: (0, path_1.join)(dest, '.eslintrc.cjs'),
                        handlePriorFile: 'skip',
                        contents: `
module.exports = {
  extends: [${configs.map((c) => `'${c}'`).join(', ')}],
  parserOptions: { tsconfigRootDir: __dirname },
  ignorePatterns: [],
  rules: {},
};
    `,
                    });
                },
            },
            {
                title: 'Add .eslintignore',
                skip: (0, creator_utils_1.hasFileUp)({ file: '.eslintignore' }) && 'Found pre-existing file',
                task: async (ctx, task) => {
                    task.output = await creator_utils_1.tasks.addFile({
                        destination: (0, path_1.join)(dest, '.eslintignore'),
                        contents: ['dist'],
                        handlePriorFile: 'skip',
                    });
                },
            },
            {
                title: 'Update package.json with eslint',
                task: async () => creator_utils_1.tasks.updatePkgJson({
                    dir: dest,
                    pkg: {
                        devDependencies: {
                            [pkg.name]: pkg.version,
                            eslint: pkg.dependencies.eslint,
                        },
                    },
                }),
            },
            {
                title: 'Adding lint npm script to package.json',
                skip: !addNpmScript,
                task: async () => creator_utils_1.tasks.updatePkgJson({
                    dir: dest,
                    pkg: {
                        scripts: {
                            lint: 'eslint ./',
                        },
                    },
                }),
            },
            {
                title: 'Adding Prettier',
                task: async (_, task) => {
                    return creator_utils_1.tasks.runSubCreator({
                        creator: creator_1.prettierConfigCreator,
                        config,
                        task,
                        knownAnswers: { addIgnore },
                    });
                },
            },
        ];
    },
});
//# sourceMappingURL=eslint-config-starter-creator.js.map