require('@rushstack/eslint-patch/modern-module-resolution');

module.exports = {
  parser: '@typescript-eslint/parser',
  plugins: [
    'prettier',
    'jsdoc',
    'promise',
    '@typescript-eslint',
    'unused-imports',
  ],
  extends: [
    'airbnb/base',
    'prettier',
    'plugin:react/jsx-runtime',
    // 'plugin:@typescript-eslint/recommended',
    //    "plugin:promise/recommended" @todo turn this back and .then() fix every error
  ],
  ignorePatterns: [
    'node_modules',
    'dist',
    '*.log',
    '*/Caches/typescript/**',
    'temp',
    '.rush',
    '!*.', // Do NOT ignore config files
  ],
  globals: {
    document: true,
    window: true,
    fetch: true,
  },
  settings: {
    'import/resolver': {
      node: {
        extensions: ['.js', '.jsx', '.ts', '.tsx'],
      },
      typescript: {},
      // use an array of glob patterns
      // typescript: {
      //   alwaysTryTypes: true, // always try to resolve types under `<root/>@types` directory even it doesn't contain any source code, like `@types/unist`
      //   // can glob
      //   directory: ['./tsconfig.base.json'],
      // },
    },
    'import/parsers': {
      '@typescript-eslint/parser': ['.ts', '.tsx'],
    },
  },
  overrides: [
    {
      // enable the rule specifically for TypeScript files
      files: ['*.ts', '*.tsx'],
      extends: ['plugin:@typescript-eslint/recommended'],
      rules: {
        '@typescript-eslint/no-explicit-any': [0],
        // '@typescript-eslint/method-signature-style': [2, 'property'],
        '@typescript-eslint/no-unused-vars': [0],
        '@typescript-eslint/explicit-function-return-type': [0],
        '@typescript-eslint/explicit-module-boundary-types': [0],
        // '@typescript-eslint/interface-name-prefix': [
        //   1,
        //   {
        //     prefixWithI: 'always',
        //     allowUnderscorePrefix: true,
        //   },
        // ],
        '@typescript-eslint/no-empty-function': [0],
        'valid-jsdoc': [0],
        'no-unused-expressions': [0],
        '@typescript-eslint/no-unused-expressions': [
          2,
          {
            allowShortCircuit: true,
            allowTernary: true,
            enforceForJSX: true,
          },
        ],
        'no-shadow': 'off',
        '@typescript-eslint/no-shadow': [
          'error',
          {
            ignoreTypeValueShadow: true,
            ignoreFunctionTypeParameterNameValueShadow: true,
          },
        ],
        '@typescript-eslint/no-inferrable-types': [2],
        '@typescript-eslint/ban-types': [
          'error',
          {
            types: {
              String: {
                message: 'Use string instead',
                fixWith: 'string',
              },
              Boolean: {
                message: 'Use boolean instead',
                fixWith: 'boolean',
              },
              Number: {
                message: 'Use number instead',
                fixWith: 'number',
              },
              Symbol: {
                message: 'Use symbol instead',
                fixWith: 'symbol',
              },

              Function: {
                message: [
                  'The `Function` type accepts any function-like value.',
                  'It provides no type safety when calling the function, which can be a common source of bugs.',
                  'It also accepts things like class declarations, which will throw at runtime as they will not be called with `new`.',
                  'If you are expecting the function to accept certain arguments, you should explicitly define the function shape.',
                ].join('\n'),
              },

              // object typing
              Object: {
                message: [
                  'The `Object` type actually means "any non-nullish value", so it is marginally better than `unknown`.',
                  '- If you want a type meaning "any object", you probably want `Record<string, unknown>` instead.',
                  '- If you want a type meaning "any value", you probably want `unknown` instead.',
                ].join('\n'),
              },
              '{}': {
                message: [
                  '`{}` actually means "any non-nullish value".',
                  '- If you want a type meaning "any object", you probably want `Record<string, unknown>` instead.',
                  '- If you want a type meaning "any value", you probably want `unknown` instead.',
                ].join('\n'),
                fixWith: 'Record<string, unknown>',
              },
              object: {
                message: [
                  'The `object` type is currently hard to use ([see this issue](https://github.com/microsoft/TypeScript/issues/21732)).',
                  'Consider using `Record<string, unknown>` instead, as it allows you to more easily inspect and use the keys.',
                ].join('\n'),
                fixWith: 'Record<string, unknown>',
              },
            },
          },
        ],
      },
    },
  ],
  rules: {
    'unused-imports/no-unused-imports': 'error',
    'unused-imports/no-unused-vars': [
      'warn',
      {
        vars: 'all',
        varsIgnorePattern: '^_',
        args: 'after-used',
        argsIgnorePattern: '^_',
      },
    ],
    'no-unused-vars': [0],
    'no-continue': [0],
    'no-use-before-define': 'off',
    '@typescript-eslint/no-use-before-define': ['error'],
    'no-restricted-syntax': [0],
    'arrow-body-style': [0],
    'no-await-in-loop': [0],
    'no-constant-condition': [1, { checkLoops: false }],
    'consistent-return': [0],
    camelcase: [0],
    'class-methods-use-this': [0],
    // 'no-use-before-define': 'off',
    // '@typescript-eslint/no-use-before-define': ['error'],
    'import/no-extraneous-dependencies': [0],
    'import/prefer-default-export': [0],
    'import/dynamic-import-chunkname': [0],
    'import/no-dynamic-require': [0],
    'import/extensions': [0],
    'no-unreachable': [0],
    'import/no-unresolved': [2],
    'jsdoc/check-types': 'error',
    'no-console': [0],
    'default-case': [0],
    'no-inner-declarations': [0],
    'no-param-reassign': [
      1,
      {
        props: false,
      },
    ],
    'no-plusplus': [
      'error',
      {
        allowForLoopAfterthoughts: true,
      },
    ],
    'no-useless-constructor': 'off',
    'no-promise-executor-return': [0],
    'lines-between-class-members': [
      'error',
      'always',
      { exceptAfterSingleLine: true },
    ],
    'prettier/prettier': ['error'],
    'no-underscore-dangle': [0],
    'prefer-arrow-callback': [0],
    strict: [0],
    // https://eslint.org/docs/rules/no-restricted-imports
    'no-restricted-imports': [
      'error',
      {
        paths: [
          {
            name: '@knapsack/app/dist/lib/constants',
            message: 'Please use `./src/constants`',
            importNames: ['*', '!apiUrlBase'],
          },
          {
            name: 'moment',
            message:
              'Please use the data-fns package (https://date-fns.org/) instead of the moment package to manipulate time and dates.',
          },
          {
            name: 'lodash',
            message:
              "Import lodash methods from default exports in subfolders, eg `import thing from 'lodash/thing'",
          },
        ],
        // To import anything from a domain, it should be exported from its `domains/X/index.ts`
        patterns: [
          // No lodash per-method packages. Use `import thing from 'lodash/thing' instead
          'lodash.*',
        ],
      },
    ],
  },
};
